﻿//*****************************************************************************
//
//  By The Dummy Programmer
//  https://www.thedummyprogrammer.com
//
//*****************************************************************************
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Text.RegularExpressions;
using System.Globalization;

namespace TDP.BaseServices.Infrastructure.DataValidation
{
    public class DataValidationHelper
    {
        CultureInfo _CultureInfo;

        public DataValidationHelper()
        {
            _CultureInfo = System.Globalization.CultureInfo.InvariantCulture;
        }

        public DataValidationHelper(CultureInfo cultureInfo)
        {
            _CultureInfo = cultureInfo;
        }

        // Instance methods
        public bool IsEmptyStringI(string value)
        {
            return string.IsNullOrEmpty(value);
        }

        public bool IsIntegerI(string value, int length, int minValue, int maxValue)
        {
            if (value.Length > length)
                return false;

            if (!Regex.Match(value, @"^\d+$").Success)
                return false;

            int v = int.Parse(value);
            if (v < minValue || v > maxValue)
                return false;

            return true;
        }

        public bool IsLong(string value, int length, long minValue, long maxValue)
        {
            if (value.Length > length)
                return false;

            if (!Regex.Match(value, @"^\d+$").Success)
                return false;

            long v = long.Parse(value);
            if (v < minValue || v > maxValue)
                return false;

            return true;
        }

        public bool IsValidDecimal(string value, int length, decimal minValue, decimal maxValue)
        {
            if (value.Length > length)
                return false;
            
            decimal V;
            bool IsValid = decimal.TryParse(value, 
                                                NumberStyles.AllowDecimalPoint | NumberStyles.AllowLeadingSign, 
                                                _CultureInfo, out V);

            if (!IsValid)
                return false;

            if (V < minValue || V > maxValue)
                return false;

            return true;
        }

        public bool IsValidFloat(string value, int length, float minValue, float maxValue)
        {
            if (value.Length > length)
                return false;

            float V;
            bool IsValid = float.TryParse(value,
                                                NumberStyles.AllowDecimalPoint | NumberStyles.AllowLeadingSign,
                                                _CultureInfo, out V);

            if (!IsValid)
                return false;

            if (V < minValue || V > maxValue)
                return false;

            return true;
        }

        public bool IsValidDouble(string value, int length, double minValue, double maxValue)
        {
            if (value.Length > length)
                return false;

            double V;
            bool IsValid = double.TryParse(value,
                                                NumberStyles.AllowDecimalPoint | NumberStyles.AllowLeadingSign,
                                                _CultureInfo, out V);

            if (!IsValid)
                return false;

            if (V < minValue || V > maxValue)
                return false;

            return true;
        }

        public bool IsValidEMailI(string email)
        {
            // This code was taken from:
            // https://docs.microsoft.com/it-it/dotnet/standard/base-types/how-to-verify-that-strings-are-in-valid-email-format

            if (string.IsNullOrWhiteSpace(email))
                return false;

            try
            {
                // Normalize the domain
                email = Regex.Replace(email, @"(@)(.+)$", DomainMapper, RegexOptions.None, TimeSpan.FromMilliseconds(200));

                // Examines the domain part of the email and normalizes it.
                string DomainMapper(Match match)
                {
                    // Use IdnMapping class to convert Unicode domain names.
                    var idn = new IdnMapping();

                    // Pull out and process domain name (throws ArgumentException on invalid)
                    string domainName = idn.GetAscii(match.Groups[2].Value);

                    return match.Groups[1].Value + domainName;
                }
            }
            catch (RegexMatchTimeoutException)
            {
                return false;
            }
            catch (ArgumentException)
            {
                return false;
            }

            try
            {
                return Regex.IsMatch(email, @"^[^@\s]+@[^@\s]+\.[^@\s]+$", RegexOptions.IgnoreCase, TimeSpan.FromMilliseconds(250));
            }
            catch (RegexMatchTimeoutException)
            {
                return false;
            }
        }

        public bool IsValidSqlServerDate(string value, DateTime minValue, DateTime maxValue)
        {
            DateTime V;
            bool IsValid = DateTime.TryParseExact(value, "yyyyMMdd", 
                                                    _CultureInfo, DateTimeStyles.None, out V);

            if (IsValid)
                return true;

            if (!IsValid)
            {
                IsValid = DateTime.TryParseExact(value, "yyyy-MM-dd",
                                                    _CultureInfo, DateTimeStyles.None, out V);
            }

            if (IsValid)
                return true;

            if (!IsValid)
            {
                IsValid = DateTime.TryParseExact(value, "yyyy-MM-ddTHH:mm",
                                                    _CultureInfo, DateTimeStyles.None, out V);
            }

            if (IsValid)
                return true;

            if (!IsValid)
            {
                IsValid = DateTime.TryParseExact(value, "yyyy-MM-ddTHH:mm:ss",
                                                    _CultureInfo, DateTimeStyles.None, out V);
            }

            if (IsValid)
                return true;


            if (V < minValue || V > maxValue)
                return false;

            return true;
        }

        public bool IsValidBool(string value)
        {
            bool V;
            bool IsValid = bool.TryParse(value, out V);

            return IsValid;
        }


        // Static methods
        public static bool IsEmptyString(string value)
        {
            return string.IsNullOrEmpty(value);
        }

        public static bool IsInteger(string value, int length, int minValue, int maxValue)
        {
            if (value.Length > length)
                return false;

            if (!Regex.Match(value, @"^\d+$").Success)
                return false;

            int v = int.Parse(value);
            if (v < minValue || v > maxValue)
                return false;

            return true;
        }

        public static bool IsValidEMail(string email)
        {
            // This code was taken from:
            // https://docs.microsoft.com/it-it/dotnet/standard/base-types/how-to-verify-that-strings-are-in-valid-email-format

            if (string.IsNullOrWhiteSpace(email))
                return false;

            try
            {
                // Normalize the domain
                email = Regex.Replace(email, @"(@)(.+)$", DomainMapper, RegexOptions.None, TimeSpan.FromMilliseconds(200));

                // Examines the domain part of the email and normalizes it.
                string DomainMapper(Match match)
                {
                    // Use IdnMapping class to convert Unicode domain names.
                    var idn = new IdnMapping();

                    // Pull out and process domain name (throws ArgumentException on invalid)
                    string domainName = idn.GetAscii(match.Groups[2].Value);

                    return match.Groups[1].Value + domainName;
                }
            }
            catch (RegexMatchTimeoutException)
            {
                return false;
            }
            catch (ArgumentException)
            {
                return false;
            }

            try
            {
                return Regex.IsMatch(email, @"^[^@\s]+@[^@\s]+\.[^@\s]+$", RegexOptions.IgnoreCase, TimeSpan.FromMilliseconds(250));
            }
            catch (RegexMatchTimeoutException)
            {
                return false;
            }
        }
    }
}
