﻿//*****************************************************************************
//
//  By The Dummy Programmer
//  https://www.thedummyprogrammer.com
//
//*****************************************************************************

using System;
using System.Security.Cryptography;

namespace TDP.BaseServices.Infrastructure.Security
{
    [Serializable]
    public struct AsymmetricCryptographyKey 
    {
        private RSAParameters _RSAKey;

        public AsymmetricCryptographyKey(CspParameters cspParameters, bool persistKey = false)
        {
            using (var Rsa = new RSACryptoServiceProvider(cspParameters))
            {
                try
                {
                    _RSAKey = Rsa.ExportParameters(true);
                }
                finally
                {
                    Rsa.PersistKeyInCsp = persistKey;
                }
            }
        }

        public AsymmetricCryptographyKey(RSAParameters key)
        {
            _RSAKey = key;
        }

        public AsymmetricCryptographyKey(string xmlKey)
        {
            using (var Rsa = new RSACryptoServiceProvider())
            {
                try
                {
                    // Import the private key used for signing the message
                    Rsa.FromXmlString(xmlKey);
                    _RSAKey = Rsa.ExportParameters(true);
                }
                finally
                {
                    // Set the keycontainer to be cleared when rsa is garbage collected.
                    Rsa.PersistKeyInCsp = false;
                }
            }
        }

        public AsymmetricCryptographyKey GetPublicKey()
        {
            RSAParameters RSAPublicKey = new RSAParameters();
            RSAPublicKey.Exponent = _RSAKey.Exponent;
            RSAPublicKey.Modulus = _RSAKey.Modulus;

            AsymmetricCryptographyKey PublickKey = new AsymmetricCryptographyKey(RSAPublicKey);

            return PublickKey;
        }

        public RSAParameters ToRSAParameters()
        {
            return _RSAKey;
        }
    }
}
