﻿//*****************************************************************************
//
//  By The Dummy Programmer
//  https://www.thedummyprogrammer.com
//
//  Part of this code was taken from:
//  https://docs.microsoft.com/en-us/dotnet/api/system.security.cryptography.aes
//*****************************************************************************

using System;
using System.IO;
using System.Security.Cryptography;


namespace TDP.BaseServices.Infrastructure.Security
{
    public static class SymmetricCryptography
    {

        public static byte[] CreateKey()
        {
            using (Aes AESAlg = Aes.Create())
            {
                AESAlg.GenerateKey();

                return AESAlg.Key;
            }
        }

        public static byte[] CreateIV()
        {
            using (Aes AESAlg = Aes.Create())
            {
                AESAlg.GenerateIV();

                return AESAlg.Key;
            }
        }

        public static byte[] EncryptData(byte[] data, byte[] key, byte[] iv)
        {
            // Check arguments.
            if (data == null || data.Length <= 0)
                throw new ArgumentNullException("data");
            if (key == null || key.Length <= 0)
                throw new ArgumentNullException("Key");
            if (iv == null || iv.Length <= 0)
                throw new ArgumentNullException("IV");
            byte[] Encrypted;

            // Create an Aes object
            // with the specified key and IV.
            using (Aes AESAlg = Aes.Create())
            {
                AESAlg.Key = key;
                AESAlg.IV = iv;

                // Create an encryptor to perform the stream transform.
                ICryptoTransform Encryptor = AESAlg.CreateEncryptor(AESAlg.Key, AESAlg.IV);

                // Create the streams used for encryption.
                using (MemoryStream MsEncrypt = new MemoryStream())
                {
                    using (CryptoStream CsEncrypt = new CryptoStream(MsEncrypt, Encryptor, CryptoStreamMode.Write))
                    {
                        CsEncrypt.Write(data, 0, data.Length);
                        Encrypted = MsEncrypt.ToArray();
                    }
                }
            }

            // Return the encrypted bytes from the memory stream.
            return Encrypted;
        }

        public static byte[] DecryptData(byte[] data, byte[] key, byte[] iv)
        {
            // Check arguments.
            if (data == null || data.Length <= 0)
                throw new ArgumentNullException("data");
            if (key == null || key.Length <= 0)
                throw new ArgumentNullException("Key");
            if (iv == null || iv.Length <= 0)
                throw new ArgumentNullException("IV");
            byte[] Decrypted;

            // Create an Aes object
            // with the specified key and IV.
            using (Aes AESAlg = Aes.Create())
            {
                AESAlg.Key = key;
                AESAlg.IV = iv;

                
                // Create a decryptor to perform the stream transform.
                ICryptoTransform Dencryptor = AESAlg.CreateDecryptor(AESAlg.Key, AESAlg.IV);

                // Create the streams used for decryption.
                using (MemoryStream MsDecrypt = new MemoryStream())
                {
                    using (CryptoStream CsDecrypt = new CryptoStream(MsDecrypt, Dencryptor, CryptoStreamMode.Read))
                    {
                        CsDecrypt.Read(data, 0, data.Length);
                        Decrypted = MsDecrypt.ToArray();
                    }
                }
            }

            // Return the decrypted bytes from the memory stream.
            return Decrypted;
        }
    }
}
